-- 
-- Created by SQL::Translator::Producer::PostgreSQL
-- Created on Sun Mar  1 22:01:36 2020
-- 
--
-- Table: ib_access
--
CREATE TABLE "ib_access" (
  -- Номер группы
  "gid" smallint NOT NULL,
  -- Раздел
  "fid" smallint NOT NULL,
  -- Просмотр раздела, возможность видеть его в списке на главной
  "view" character varying(1) DEFAULT '1' NOT NULL,
  -- Чтение сообщений в разделе
  "read" character varying(1) DEFAULT '1' NOT NULL,
  -- Отправка сообщений
  "post" character varying(1) DEFAULT '1' NOT NULL,
  -- Прикрепление файлов
  "attach" character varying(1) DEFAULT '1' NOT NULL,
  -- Создание тем
  "topic" character varying(1) DEFAULT '1' NOT NULL,
  -- Создание опросов
  "poll" character varying(1) DEFAULT '1' NOT NULL,
  -- Использование HTML-кода без экранирования
  "html" character varying(1) DEFAULT '0' NOT NULL,
  -- Разрешено ли голосовать в опросах в разделе
  "vote" character varying(1) DEFAULT '0' NOT NULL,
  -- Разрешено ли рейтинговать сообщения в разделе
  "rate" character varying(1) DEFAULT '0' NOT NULL,
  -- Возможность редактировать свои сообщения
  "edit" character varying(1) DEFAULT '1' NOT NULL,
  -- Возможность отправлять сообщения без ввода CAPTCHA
  "nocaptcha" character varying(1) DEFAULT '1' NOT NULL,
  -- Разрешено ли отправлять сообщения без премодерации
  "nopremod" character varying(1) DEFAULT '1' NOT NULL,
  PRIMARY KEY ("gid", "fid")
);

--
-- Table: ib_banned_ip
--
CREATE TABLE "ib_banned_ip" (
  "start" integer NOT NULL,
  "end" integer NOT NULL,
  "till" integer NOT NULL,
  PRIMARY KEY ("start", "end", "till")
);

--
-- Table: ib_bots
--
CREATE TABLE "ib_bots" (
  "id" serial NOT NULL,
  -- Часть строки User Agent, по которой определяется бот
  "user_agent" character varying(255) NOT NULL,
  -- Название бота (выводится в админке и списке присутствующих)
  "bot_name" character varying(255) NOT NULL,
  -- Время последнего визита
  "last_visit" integer NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_captcha
--

-- Comments:
-- Коды CAPTCHA для соответствующего модуля
--
CREATE TABLE "ib_captcha" (
  -- Хеш, по которому осуществляется проверка кода (он же совпадает с именем файла в files/captcha/хеш.jpg)
  "hash" character(32) NOT NULL,
  -- Код, который должен ввести пользователь
  "code" character(8) NOT NULL,
  -- Признак того, что данный код еще не использован
  "active" character varying(1) DEFAULT '1' NOT NULL,
  -- Время генерации пары хеш-код (нужно для удаления устаревших пар)
  "lastmod" bigint NOT NULL,
  -- IP адрес пользователя, запросившего хеш (возможно, будет использоваться для блокировки)
  "ip" bigint NOT NULL,
  PRIMARY KEY ("hash")
);
CREATE INDEX "lastmod" on "ib_captcha" ("lastmod");

--
-- Table: ib_category
--
CREATE TABLE "ib_category" (
  "id" serial NOT NULL,
  -- Название категории
  "title" character varying(80) NOT NULL,
  -- Если 1, то по умолчанию на главной выводится в свернутом виде
  "collapsed" character varying(1) DEFAULT '0' NOT NULL,
  "sortfield" smallint NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_complain
--

-- Comments:
-- Жалобы пользователей на сообщения
--
CREATE TABLE "ib_complain" (
  -- Номер жалобы
  "id" serial NOT NULL,
  -- Идентификатор пользователя, ее отправившего
  "uid" integer DEFAULT 0,
  -- Идентификатор сообщения, на которое подана жалоба
  "pid" integer DEFAULT 0,
  -- Жалоба обработана
  "processed" character varying(1) DEFAULT '0',
  -- Модератор, выполнивший обработку
  "moderator" integer DEFAULT 0,
  -- Текст жалобы
  "text" character varying(255) DEFAULT NULL,
  -- Комментарий модератора
  "mod_comment" character varying(255) DEFAULT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_crontab
--

-- Comments:
-- Список заданий, выполняемых по времени
--
CREATE TABLE "ib_crontab" (
  -- Идентификатор задания
  "id" serial NOT NULL,
  -- Библиотека, в которой находится выполняемая процедура
  "library" character varying(24) NOT NULL,
  -- Название процедуры
  "proc" character varying(255) NOT NULL,
  -- Параметры процедуры
  "params" character varying(255) NOT NULL,
  -- Описание (показывается администраторам сайта)
  "description" character varying(255) NOT NULL,
  -- Время след. выполнения
  "nextrun" bigint NOT NULL,
  -- Период выполнения (в минутах), если равен нулю, задание отключено
  "period" smallint NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "nextrun" on "ib_crontab" ("nextrun", "period");

--
-- Table: ib_file
--

-- Comments:
-- Приложенные файлы
--
CREATE TABLE "ib_file" (
  -- Ключ-идентификатор файла
  "fkey" character(16) DEFAULT '' NOT NULL,
  -- Номер объекта, к которому прикреплен файл
  "oid" bigint NOT NULL,
  -- Тип прикрепления: 1 -- к форумному сообщению, 2 -- к ЛС (не используется)
  "type" smallint NOT NULL,
  -- Имя файла при скачивании
  "filename" character varying(255) NOT NULL,
  -- Размер файла
  "size" bigint NOT NULL,
  "format" character varying(6) DEFAULT 'attach' NOT NULL,
  "extension" character(4) NOT NULL,
  PRIMARY KEY ("fkey")
);
CREATE INDEX "oid" on "ib_file" ("oid", "type");

--
-- Table: ib_forum
--
CREATE TABLE "ib_forum" (
  "id" serial NOT NULL,
  -- Модуль, отвечающий за показ страницы
  "module" character varying(16) NOT NULL,
  -- Название раздела
  "title" character varying(80) NOT NULL,
  -- Описание раздела
  "descr" character varying(255) DEFAULT '' NOT NULL,
  -- Частичный URL
  "hurl" character varying(255) NOT NULL,
  "category_id" smallint DEFAULT 0 NOT NULL,
  -- Родительский раздел
  "parent_id" smallint DEFAULT 0 NOT NULL,
  -- Владелец раздела (для личных блогов, галерей и т.п.)
  "owner" integer DEFAULT 0 NOT NULL,
  -- Порядок сортировки
  "sortfield" smallint DEFAULT 0 NOT NULL,
  -- Форум закрыт для новых сообщений: 0 -- нет, 1 -- да
  "locked" character varying(1) DEFAULT '0' NOT NULL,
  -- Дата последнего изменения
  "lastmod" integer DEFAULT 0 NOT NULL,
  -- Id последнего сообщения
  "last_post_id" integer DEFAULT 0 NOT NULL,
  -- Шаблон для отображения раздела
  "template" character varying(32) DEFAULT '' NOT NULL,
  -- Приоритет шаблона форума над шаблоном пользователя
  "template_override" character varying(1) DEFAULT '0' NOT NULL,
  -- Использование BBCode
  "bcode" character varying(1) DEFAULT '1' NOT NULL,
  -- Максимальное количество смайликов в сообщении
  "max_smiles" smallint DEFAULT 16 NOT NULL,
  -- Максимальное количество файлов, которое можно прикрепить к сообщению
  "max_attach" smallint DEFAULT 0 NOT NULL,
  -- Допустимые типы прикрепляемых файлов. Указываются как битовая маска: 255 -- все, 1 -- картинки, 2 -- видео, 4 -- аудио, 8 -- текст, 128 -- все остальное
  "attach_types" smallint DEFAULT 255 NOT NULL,
  -- Количество видимых тем (т.е. со статусом "0", без учета стоящих на премодерации и удаленных)
  "topic_count" integer DEFAULT 0 NOT NULL,
  -- Количество видимых сообщений
  "post_count" integer DEFAULT 0 NOT NULL,
  -- Является ли раздел статистически значимым
  "is_stats" character varying(1) DEFAULT '1' NOT NULL,
  -- Является ли флуд-разделом (не показывать в "Обновившихся" и "Непрочитанных", а так же поиске, если он не включен явно)
  "is_flood" character varying(1) DEFAULT '0' NOT NULL,
  -- Показывать ли раздел на главной странице
  "is_start" character varying(1) DEFAULT '1' NOT NULL,
  -- Имя файла при наличии новых сообщений
  "icon_new" character varying(255) DEFAULT '' NOT NULL,
  -- Имя файла при отсутствии новых сообщений
  "icon_nonew" character varying(255) DEFAULT '' NOT NULL,
  -- Порядок сортировки тем в разделе по умолчанию: по возрастанию или убыванию
  "sort_mode" character varying(4) DEFAULT 'DESC' NOT NULL,
  -- Способ сортировки тем в форуме по умолчанию: по дате последнего сообщения или дате создания
  "sort_column" character varying(14) DEFAULT 'last_post_time' NOT NULL,
  -- Опросы в темах: 0 -- отключены вообще, 1 -- разрешены, 2 -- рарешены уже созданные, но запрещено создание новых
  "polls" character varying(1) DEFAULT '1' NOT NULL,
  -- Самомодерация темы ее создателем
  "selfmod" character varying(1) DEFAULT '0' NOT NULL,
  -- Первое сообщение является приклееным (выводится на всех страницах): 0 -- нет, 1 -- выставляется модераторами, 2 -- выставлеяется пользователем при создании темы, 3 -- есть всегда
  "sticky_post" character varying(1) DEFAULT '2' NOT NULL,
  -- Разрешены ли рейтинги на форуме: 0 - нет, 1 -- да, 2 -- только положительные
  "rate" character varying(1) DEFAULT '2' NOT NULL,
  -- Количество голосов "за", после которых сообщение становится ценным
  "rate_value" float(8) DEFAULT 0 NOT NULL,
  -- Количество голосов против, после которых сообщение становится флудом
  "rate_flood" float(8) DEFAULT 0 NOT NULL,
  -- Использование тегов для тем: 0 -- нет, 1 -- да, 2 -- ставить теги могут только модераторы
  "tags" character varying(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "sortkey" on "ib_forum" ("sortfield");
CREATE INDEX "hurl" on "ib_forum" ("hurl");

--
-- Table: ib_forum_type
--

-- Comments:
-- Описание типов разделов
--
CREATE TABLE "ib_forum_type" (
  -- Имя модуля, который отвечает за отображение раздела
  "module" character varying(40) NOT NULL,
  -- Название раздела для вывода в админке
  "typename" character varying(255) NOT NULL,
  -- У раздела могут быть правила
  "has_rules" character varying(1) DEFAULT '1' NOT NULL,
  -- У раздела может быть вводное слово
  "has_foreword" character varying(1) DEFAULT '1' NOT NULL,
  -- Над данным типом разделов разрешены групповые операции
  "allow_mass" character varying(1) DEFAULT '1' NOT NULL,
  -- У разделов данного типа могут быть вложенные подразделы
  "allow_subforums" character varying(1) DEFAULT '0' NOT NULL,
  -- Разделы данного типа могут быть личными
  "allow_personal" character varying(1) DEFAULT '0' NOT NULL,
  -- Поле для сортировки
  "sortfield" smallint DEFAULT 1 NOT NULL,
  -- Шаблон для генерации правил для файла .htaccess
  "htaccess" text,
  PRIMARY KEY ("module")
);

--
-- Table: ib_group
--
CREATE TABLE "ib_group" (
  -- Номер группы, он же уровень доступа. В целом рекомендуется так: чем выше, тем статуснее группа
  "level" smallint NOT NULL,
  "name" character varying(32) NOT NULL,
  -- В особые группы пользователь может быть добавлен только явно, администратором
  "special" character varying(1) DEFAULT '1' NOT NULL,
  "floodtime" smallint DEFAULT 60 NOT NULL,
  "privmsg_hour" smallint DEFAULT 240 NOT NULL,
  -- Макс размер приложенного файла (в килобайтах)
  "max_attach" smallint NOT NULL,
  -- Минимальное число сообщений
  "min_posts" smallint DEFAULT 0 NOT NULL,
  -- Разрешено пользователю ставить себе "особое звание"
  "custom_title" character varying(1) DEFAULT '0' NOT NULL,
  -- Является ли группа администраторами
  "admin" character varying(1) DEFAULT '0' NOT NULL,
  -- Группа считается группой команды форума (из групп команды назначаются модераторы и эксперты форума, ее участники выводятся в списке "наша команда")
  "team" character varying(1) DEFAULT '0' NOT NULL,
  -- Founder -- права суперпользователя форума (администратор с правом назначения/снятия других администраторов и сменой глобальных настроек)
  "founder" character varying(1) DEFAULT '0' NOT NULL,
  -- Количество дней, которое должно пройти с момента регистрации, для получения уровня (если уровень -- не "особый"
  "min_reg_time" smallint DEFAULT 0 NOT NULL,
  -- Режим использования гиперссылок:  none -- запрещено отправлять сообщения со ссылками, premoderate -- сообщения со ссылками уходят на премодерацию, nofollow -- ссылкам ставится аттрибут nofollow, allow -- ссылки выводятся нормально
  "links_mode" character varying(8) DEFAULT 'allow' NOT NULL,
  PRIMARY KEY ("level")
);

--
-- Table: ib_last_visit
--

-- Comments:
-- Данные о посещении раздела или темы пользователем
--
CREATE TABLE "ib_last_visit" (
  -- Идентификатор раздела или темы
  "oid" integer NOT NULL,
  -- Тип объекта: раздел или тема
  "type" character varying(5) NOT NULL,
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Время текущего посещения
  "visit1" bigint NOT NULL,
  -- Время предыдущего посещения
  "visit2" bigint NOT NULL,
  -- Находится ли тема в закладках
  "bookmark" character varying(1) DEFAULT '0' NOT NULL,
  -- Подписан ли пользователь на тему (0 -- нет, 1 -- отправка уведомлений сразу, 2 -- уведомление только о первом новом сообщении, 3 и более -- периодическая высылка уведомлений)
  "subscribe" smallint DEFAULT 0 NOT NULL,
  -- Дата последней отправки сообщения
  "lastmail" bigint NOT NULL,
  -- Если пользователь писал в эту тему
  "posted" character varying(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY ("oid", "uid", "type")
);

--
-- Table: ib_log_action
--

-- Comments:
-- Лог модераторских действий с возможностью отката измеений
--
CREATE TABLE "ib_log_action" (
  "id" serial NOT NULL,
  -- Номер сообщения, над которым совершалось действие
  "pid" integer DEFAULT 0 NOT NULL,
  -- Номер темы, над которой совершалось действие
  "tid" integer DEFAULT 0 NOT NULL,
  -- Номер раздела, над которым совершалось действие
  "fid" integer DEFAULT 0 NOT NULL,
  -- Тип модераторского действия
  "type" smallint NOT NULL,
  -- Время модераторского действия
  "time" integer NOT NULL,
  -- Модератор или пользователь, выполнивший действие
  "uid" integer NOT NULL,
  -- Данные для отката модераторского действия
  "data" text NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_mark_all
--

-- Comments:
-- Данные об отметке раздела или всего форума как прочтенного
--
CREATE TABLE "ib_mark_all" (
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Раздел, для которого выполняется действие "Отметить все" (0 -- весь форум)
  "fid" integer NOT NULL,
  -- Время выполнения действия
  "mark_time" integer NOT NULL,
  PRIMARY KEY ("uid", "fid")
);

--
-- Table: ib_menu
--

-- Comments:
-- Список различных меню, используемых движком
--
CREATE TABLE "ib_menu" (
  -- Идентификатор меню
  "id" serial NOT NULL,
  -- Описание меню (показывается в админцентре)
  "descr" character varying(255) NOT NULL,
  -- Запрет на удаление меню (для наиболее важных системных меню)
  "locked" character varying(1) NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_menu_item
--

-- Comments:
-- Элементы меню
--
CREATE TABLE "ib_menu_item" (
  -- Идентификатор пункта
  "id" serial NOT NULL,
  -- Идентификатор меню, которому принадлежит пункт
  "mid" smallint NOT NULL,
  -- Заголовок пункта меню (выводится при выводе меню)
  "title" character varying(255) NOT NULL,
  -- URL пункта меню
  "url" character varying(255) NOT NULL,
  -- Поле для определения порядка сортировки
  "sortfield" smallint NOT NULL,
  -- Показывать пункт гостям
  "show_guests" character varying(1) DEFAULT '0' NOT NULL,
  -- Показывать пункт пользователям
  "show_users" character varying(1) DEFAULT '0' NOT NULL,
  -- Показывать пункт администраторам
  "show_admins" character varying(1) DEFAULT '0' NOT NULL,
  -- Является ли ссылка адресом относительно корня форума (нужно ли применять функцию url)
  "hurl_mode" character varying(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "mid" on "ib_menu_item" ("mid", "sortfield");

--
-- Table: ib_moderator
--

-- Comments:
-- Данные о модераторах и экспертах форума
--
CREATE TABLE "ib_moderator" (
  -- Объект, для которого пользователь показывается как член команды
  "fid" integer NOT NULL,
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Роль (модератор или эксперт)
  "role" character varying(9) NOT NULL,
  PRIMARY KEY ("fid", "uid", "role")
);

--
-- Table: ib_online
--
CREATE TABLE "ib_online" (
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Хеш данных пользователя (IP-адрес, user agent и еще несколько полей заголовков HTTP)
  "hash" character(32) NOT NULL,
  -- Время последнего визита
  "visittime" integer NOT NULL,
  -- Тип пользователя (0 -- гость, -1 -- зарегистрированный пользователь, -2 -- член команды, положительные значения -- номера ботов
  "type" smallint NOT NULL,
  -- Номер раздела, над которым совершается действие
  "fid" integer NOT NULL,
  -- Номер темы
  "tid" integer NOT NULL,
  -- Описание действия, обрабатыавется через sprintf с подстановкой ссылок на тему и форум
  "text" character varying(255) NOT NULL,
  -- IP пользователя (подумать, нужен ли вообще)
  "ip" character varying(40) NOT NULL,
  PRIMARY KEY ("hash")
);
CREATE INDEX "uid_online" on "ib_online" ("uid");

--
-- Table: ib_poll
--

-- Comments:
-- Таблица опросов
--
CREATE TABLE "ib_poll" (
  -- Тема, к которой привязан опрос
  "id" integer NOT NULL,
  -- Текст вопроса
  "question" character varying(255) NOT NULL,
  -- Дата окончания опроса
  "endtime" integer NOT NULL,
  -- Режим голосования: 0 -- анонимный, 1 -- с возможностью просмотра проголосовавших (пока не используется)
  "mode" character varying(1) DEFAULT '0' NOT NULL,
  -- Количество вариантов ответов, которое можно выбрать (пока не используется)
  "max_variants" smallint DEFAULT 1 NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_poll_variant
--
CREATE TABLE "ib_poll_variant" (
  -- Идентификатор варианта ответа
  "id" serial NOT NULL,
  -- Идентификатор темы с голосованием
  "tid" integer NOT NULL,
  -- Текст варианта ответа
  "text" character varying(80) NOT NULL,
  -- Количество голосований за этот вариант
  "count" integer DEFAULT 0 NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "poll_tid" on "ib_poll_variant" ("tid");

--
-- Table: ib_post
--

-- Comments:
-- Данные о сообщениях в темах и разделах
--
CREATE TABLE "ib_post" (
  "id" serial NOT NULL,
  -- Тема, к которой относится сообщение
  "tid" integer NOT NULL,
  -- Номер владельца сообщения
  "uid" integer DEFAULT 1 NOT NULL,
  -- Автор сообщения (NULL -- если зарегистрированный пользователь)
  "author" character varying(64) DEFAULT 'Guest' NOT NULL,
  -- Дата отправки сообщения
  "postdate" bigint DEFAULT 0 NOT NULL,
  -- Количество редактирований
  "editcount" smallint DEFAULT 0 NOT NULL,
  -- Идентификатор последнего отредактировавшего сообщение
  "editor_id" integer DEFAULT 0 NOT NULL,
  -- Ценность сообщения: 0 -- обычное, 1 -- ценное, -1 -- флуд
  "value" character varying(2) DEFAULT '0' NOT NULL,
  -- Статус сообщения: 0 -- нормальное сообщение, 1 -- на премодерации, 2 -- удалено
  "status" character varying(1) DEFAULT '0' NOT NULL,
  -- Сообщение заблокировано от редактирования
  "locked" character varying(1) DEFAULT '0' NOT NULL,
  -- В сообщении разрешено использование HTML
  "html" character varying(1) DEFAULT '0' NOT NULL,
  -- В сообщении разрешено использование BoardCode
  "bcode" character varying(1) DEFAULT '1' NOT NULL,
  -- В сообщении разрешены графические смайлики
  "smiles" character varying(1) DEFAULT '1' NOT NULL,
  -- Автоматически обрабатывать ссылки
  "links" character varying(1) DEFAULT '1' NOT NULL,
  -- Автоматически типографировать сообщение
  "typograf" character varying(1) DEFAULT '1' NOT NULL,
  -- Рейтинг сообщения
  "rating" float(8) DEFAULT 0 NOT NULL,
  -- IP, с которого было отправлено сообщение
  "ip" bigint NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "topic" on "ib_post" ("tid", "postdate");
CREATE INDEX "author_uid" on "ib_post" ("uid");

--
-- Table: ib_privmsg_link
--

-- Comments:
-- Связь сообщений в теме и пользователей, у которых их видно
--
CREATE TABLE "ib_privmsg_link" (
  "pm_id" bigint NOT NULL,
  "uid" integer NOT NULL,
  PRIMARY KEY ("uid", "pm_id")
);

--
-- Table: ib_privmsg_post
--

-- Comments:
-- Личные сообщения 
--
CREATE TABLE "ib_privmsg_post" (
  "id" serial NOT NULL,
  "pm_thread" integer NOT NULL,
  -- Количество получателей сообщения
  "subscribers" smallint DEFAULT 1,
  "uid" integer NOT NULL,
  "text" text NOT NULL,
  "postdate" integer NOT NULL,
  "html" character varying(1) DEFAULT '0' NOT NULL,
  "bcode" character varying(1) DEFAULT '1' NOT NULL,
  "smiles" character varying(1) DEFAULT '1' NOT NULL,
  "links" character varying(1) DEFAULT '1' NOT NULL,
  "typograf" character varying(1) DEFAULT '1' NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "thread" on "ib_privmsg_post" ("pm_thread", "postdate");

--
-- Table: ib_privmsg_thread
--

-- Comments:
-- Таблица тем ЛС.
--
CREATE TABLE "ib_privmsg_thread" (
  "id" serial NOT NULL,
  -- Название темы ЛС
  "title" character varying(255) NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_privmsg_thread_user
--

-- Comments:
-- Данные о теме, специфичные для пользователя
--
CREATE TABLE "ib_privmsg_thread_user" (
  "pm_thread" bigint NOT NULL,
  "uid" integer NOT NULL,
  -- Количество доступных сообщений
  "total" smallint DEFAULT 0 NOT NULL,
  -- Количество непрочитанных сообщений
  "unread" smallint DEFAULT 0 NOT NULL,
  -- Идентификатор последнего доступного пользователю сообщения
  "last_post_date" integer DEFAULT 0 NOT NULL,
  PRIMARY KEY ("uid", "pm_thread")
);

--
-- Table: ib_rating
--

-- Comments:
-- Информация о том, когда было отрейтинговано сообщение
--
CREATE TABLE "ib_rating" (
  "id" integer NOT NULL,
  "uid" integer NOT NULL,
  "value" float(8) NOT NULL,
  "time" integer NOT NULL,
  "ip" integer NOT NULL,
  PRIMARY KEY ("id", "uid")
);

--
-- Table: ib_relation
--
CREATE TABLE "ib_relation" (
  "from" serial NOT NULL,
  "to" integer NOT NULL,
  "type" character varying(6) NOT NULL,
  PRIMARY KEY ("from", "to")
);

--
-- Table: ib_search
--

-- Comments:
-- Таблица для хранения результатов поиска FULLTEXT или Sphynx
--
CREATE TABLE "ib_search" (
  -- Номер поиска
  "id" serial NOT NULL,
  -- Id ищущего пользователя
  "owner" integer DEFAULT 0,
  -- Режим вывода (темы или сообщения)
  "output_mode" character varying(6) DEFAULT 'posts',
  -- Режим поиска (для выдачи информации о запросе при его показе)
  "search_type" smallint DEFAULT 1,
  -- Текст запроса
  "query" character varying(255) DEFAULT '1',
  -- Время запроса
  "time" integer DEFAULT NULL,
  -- Расширенные данные поиска: дата, время, разделы
  "extdata" text,
  PRIMARY KEY ("id")
);

--
-- Table: ib_search_result
--

-- Comments:
-- Таблица для хранения результатов поиска
--
CREATE TABLE "ib_search_result" (
  -- Номер поиска
  "sid" integer NOT NULL,
  -- Номер найденного сообщения или темы
  "oid" integer NOT NULL,
  -- Релевантность сообщения (или значение для сортировки)
  "relevancy" float(8) DEFAULT 0,
  PRIMARY KEY ("oid", "sid")
);
CREATE INDEX "relevancy" on "ib_search_result" ("sid", "relevancy");

--
-- Table: ib_smile
--
CREATE TABLE "ib_smile" (
  -- Код смайлика
  "code" character varying(16) NOT NULL,
  -- Имя файла со смайликом
  "file" character varying(255) NOT NULL,
  -- Описание того, что смайлик обозначает
  "descr" character varying(255) NOT NULL,
  -- Режим отображения смайлика в панели редактора: more -- обычный, dropdown -- в выпадающем списке, hidden -- не отображать
  "mode" character varying(8) DEFAULT 'dropdown' NOT NULL,
  -- Поле для сортировки
  "sortfield" smallint DEFAULT 0 NOT NULL,
  PRIMARY KEY ("code")
);

--
-- Table: ib_subactions
--

-- Comments:
-- Вспомогательные действия (пока не используется)
--
CREATE TABLE "ib_subactions" (
  -- Имя модуля. Может содержать одно из зарезервированных ключевых слов: All (все модули), Forum (модуль форумного типа в режиме показа раздела), Topic (модуль форумного типа в режиме показа темы)
  "module" character varying(32) DEFAULT '''All''' NOT NULL,
  -- Имя основного действия, при котором выполняется данное вспомогательное. Может быть *, если нужно выполнить для всех действий.
  "action" character varying(32) DEFAULT '*' NOT NULL,
  -- Имя библиотеки, в которой лежит обработчик вспомогательного действия
  "library" character varying(32) NOT NULL,
  -- Имя функции в библиотеке, реализующей данное действие
  "function" character varying(32) NOT NULL,
  -- Признак того, что действие включено и должно выполняться
  "active" character varying(1) DEFAULT '1' NOT NULL,
  -- Параметры, передаваемые в функцию
  "params" character varying(255) NOT NULL,
  -- Приоритет по умолчанию (более высокие выполняются первыми)
  "priority" smallint DEFAULT 0 NOT NULL,
  PRIMARY KEY ("module", "action")
);

--
-- Table: ib_tagentry
--
CREATE TABLE "ib_tagentry" (
  "tag_id" smallint NOT NULL,
  "item_id" integer NOT NULL,
  PRIMARY KEY ("tag_id", "item_id")
);

--
-- Table: ib_tagname
--
CREATE TABLE "ib_tagname" (
  "id" serial NOT NULL,
  "type" character varying(5) NOT NULL,
  "tagname" character varying(32) NOT NULL,
  "count" integer DEFAULT NULL,
  PRIMARY KEY ("id"),
  CONSTRAINT "tagname" UNIQUE ("tagname", "type")
);

--
-- Table: ib_text
--
CREATE TABLE "ib_text" (
  -- Номер раздела
  "id" integer NOT NULL,
  -- Тип текста: 0 -- правила, 1 -- объявление, 2 -- текст статического раздела
  "type" smallint NOT NULL,
  -- Текст
  "data" text NOT NULL,
  "tx_lastmod" integer DEFAULT 0 NOT NULL,
  PRIMARY KEY ("id", "type")
);

--
-- Table: ib_timeout
--

-- Comments:
-- Таблица хранения таймаутов между действиями (типа регистраци
--
CREATE TABLE "ib_timeout" (
  -- Время последней попытки совершения действия
  "time" bigint NOT NULL,
  -- Условное название действия
  "action" character varying(32) NOT NULL,
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  "ip" bigint NOT NULL
);
CREATE INDEX "time" on "ib_timeout" ("time", "action", "uid");

--
-- Table: ib_topic
--
CREATE TABLE "ib_topic" (
  "id" serial NOT NULL,
  -- Идентификатор раздела, в котором находится тема
  "fid" integer NOT NULL,
  -- Название темы
  "title" character varying(80) NOT NULL,
  -- Описание темы
  "descr" character varying(255) NOT NULL,
  -- Статус темы: 0 -- нормальная, 1 -- на премодерации, 2 -- удалена
  "status" character varying(1) DEFAULT '0' NOT NULL,
  -- HURL темы (без HURL раздела)
  "hurl" character varying(255) DEFAULT '' NOT NULL,
  -- Тема закрыта
  "locked" character varying(1) DEFAULT '0' NOT NULL,
  -- Первое сообщение темы
  "first_post_id" integer DEFAULT 0 NOT NULL,
  -- Второе сообщение темы
  "last_post_id" integer DEFAULT 0 NOT NULL,
  -- Время последнего изменения темы (отправки сообщения, редактирования или модерации)
  "lastmod" integer NOT NULL,
  -- Отображаемое количество сообщений
  "post_count" smallint DEFAULT 0 NOT NULL,
  -- Количество сообщений, помеченных как флуд
  "flood_count" smallint DEFAULT 0 NOT NULL,
  -- Количество ценных сообщений
  "valued_count" smallint DEFAULT 0 NOT NULL,
  -- Владелец темы (обычно автор первого сообщения)
  "owner" integer DEFAULT 0 NOT NULL,
  -- Тема является прикленной
  "sticky" character varying(1) DEFAULT '0' NOT NULL,
  -- Показывать ли первое сообщение темы на каждой странице
  "sticky_post" character varying(1) DEFAULT '0' NOT NULL,
  -- Тема есть в "лучших темах форума"
  "favorites" character varying(1) DEFAULT '0' NOT NULL,
  -- Расширенный статус (используется специализированными разделами)
  "ext_status" integer DEFAULT 0 NOT NULL,
  -- Время отправки последнего сообщения (сделано в целях оптимизации нагрузки на БД)
  "last_post_time" integer DEFAULT 0 NOT NULL,
  -- Суммарный рейтинг темы
  "rating" integer DEFAULT 0 NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "forum" on "ib_topic" ("fid", "last_post_id");

--
-- Table: ib_user
--
CREATE TABLE "ib_user" (
  "id" serial NOT NULL,
  "login" character varying(32) NOT NULL,
  "password" character varying(255) NOT NULL,
  "pass_crypt" smallint NOT NULL,
  "title" character varying(80) DEFAULT '' NOT NULL,
  "gender" character varying(1) DEFAULT 'U' NOT NULL,
  "birthdate" date DEFAULT NULL,
  "location" character varying(80) NOT NULL,
  "status" smallint DEFAULT 0 NOT NULL,
  "canonical" character varying(255) NOT NULL,
  "signature" character varying(255) NOT NULL,
  "rnd" integer NOT NULL,
  "display_name" character varying(32) NOT NULL,
  "avatar" character varying(4) DEFAULT 'none' NOT NULL,
  "photo" character varying(4) DEFAULT 'none' NOT NULL,
  "email" character varying(255) NOT NULL,
  -- Реальное имя пользователя, если он захочет его указать
  "real_name" character varying(255) DEFAULT '' NOT NULL,
  PRIMARY KEY ("id"),
  CONSTRAINT "login" UNIQUE ("login"),
  CONSTRAINT "display_name" UNIQUE ("display_name")
);
CREATE INDEX "location" on "ib_user" ("location");

--
-- Table: ib_user_award
--
CREATE TABLE "ib_user_award" (
  -- Идентификатор
  "id" serial NOT NULL,
  -- Идентификатор награжденного пользователя
  "uid" integer NOT NULL,
  -- Имя файла с изображением награды
  "file" character varying(255) NOT NULL,
  -- Описание причины для награждения
  "descr" character varying(255) NOT NULL,
  -- Дата награждения
  "time" integer NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "uid_uaw" on "ib_user_award" ("uid");

--
-- Table: ib_user_contact
--
CREATE TABLE "ib_user_contact" (
  "uid" integer NOT NULL,
  "cid" smallint NOT NULL,
  "value" character varying(80) NOT NULL,
  PRIMARY KEY ("uid", "cid")
);

--
-- Table: ib_user_contact_type
--
CREATE TABLE "ib_user_contact_type" (
  "cid" serial NOT NULL,
  -- Название контакта или социальной сети
  "c_title" character varying(80) NOT NULL,
  -- URL значка контакта или социальной сети
  "icon" character varying(255) NOT NULL,
  -- Ссылка на контакт или профиль соцсети
  "link" character varying(255) NOT NULL,
  -- Поле для сортировки
  "c_sort" smallint NOT NULL,
  -- Идентификатор для библиотеки авторизации через соцсети
  "c_name" character varying(32) NOT NULL,
  -- Нужна ли проверка на наличие прав размещать ссылки при выводе этого контакта
  "c_permission" character varying(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY ("cid")
);

--
-- Table: ib_user_ext
--
CREATE TABLE "ib_user_ext" (
  "id" integer NOT NULL,
  -- Количество сообщений
  "post_count" integer DEFAULT 0 NOT NULL,
  -- Суммарный рейтинг
  "rating" float(8) DEFAULT 0 NOT NULL,
  -- Сумма баллов предупреждений
  "warnings" smallint DEFAULT 0 NOT NULL,
  -- Баланс (сейчас не используется)
  "balance" numeric(10,0) DEFAULT 0 NOT NULL,
  -- Если пользователь изгнан, дата окончания срока
  "banned_till" integer DEFAULT 0 NOT NULL,
  -- Группа прав доступа пользователя
  "group_id" smallint DEFAULT 0 NOT NULL,
  -- Дата регистрации
  "reg_date" integer DEFAULT 0 NOT NULL,
  -- IP, с которого произведена регистрация
  "reg_ip" character varying(40)  DEFAULT 0 NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "user_group" on "ib_user_ext" ("group_id");

--
-- Table: ib_user_field
--

-- Comments:
-- Задаваемые поля для профиля пользователя
--
CREATE TABLE "ib_user_field" (
  -- Идентификатор поля
  "id" serial NOT NULL,
  -- Название поля
  "title" character varying(60) NOT NULL,
  -- Тип поля: текст, числовое значение, выбор значения из списка типа Select или переключателей Radio
  "type" character varying(11) DEFAULT 'text' NOT NULL,
  -- Список возможных значений для select и radio, для text -- регулярное выражение для проверки корректности ввода
  "values" text NOT NULL,
  -- Выводить значение поля при показе сообщений пользователя или только в его профиле
  "in_msg" character varying(1) DEFAULT '1' NOT NULL,
  -- Поле для сортировки
  "sortfield" smallint NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_user_settings
--
CREATE TABLE "ib_user_settings" (
  "id" integer NOT NULL,
  -- Тем на странице
  "topics_per_page" smallint DEFAULT 10 NOT NULL,
  -- Сообщений на странице
  "posts_per_page" smallint DEFAULT 20 NOT NULL,
  -- Используемый шаблон
  "template" character varying(20) DEFAULT '' NOT NULL,
  -- Порядок сортировки сообщений в теме
  "msg_order" character varying(4) DEFAULT 'ASC' NOT NULL,
  -- Подписка на обновления: нет, только на созданные темы, на все темы, в которых пользователь пишет ответ
  "subscribe" character varying(4) DEFAULT 'None' NOT NULL,
  -- Часовой пояс участника (смещение в секундах)
  "timezone" smallint DEFAULT 10800 NOT NULL,
  -- Показывать подписи
  "signatures" character varying(1) DEFAULT '1' NOT NULL,
  -- Показывать аватары
  "avatars" character varying(1) DEFAULT '1' NOT NULL,
  -- Показывать смайлики
  "smiles" character varying(1) DEFAULT '1' NOT NULL,
  -- Показывать прикрепленные и вставленные изображения
  "pics" character varying(1) DEFAULT '1' NOT NULL,
  -- Сворачивать длинные сообщения: 0 -- никогда, 1 -- да, 2 -- только  помеченные как флуд
  "longposts" character varying(1) DEFAULT '0' NOT NULL,
  -- Показывать дату рождения (0 -- нет, 1 -- да, 2 -- только дату, 3 -- только возраст)
  "show_birthdate" character varying(1) DEFAULT '3' NOT NULL,
  -- Режим рассылки уведомлений
  "subscribe_mode" smallint DEFAULT 0 NOT NULL,
  -- Отправлять полный текст сообщения на почту
  "email_fulltext" character varying(1) DEFAULT '1' NOT NULL,
  -- Отправлять увеедомления о новых личных сообщениях
  "email_pm" character varying(1) DEFAULT '1' NOT NULL,
  -- Разрешить отправку сообщений через форму на сайте
  "email_message" character varying(1) DEFAULT '1' NOT NULL,
  -- Получать рассылки от администратора
  "email_broadcasts" character varying(1) DEFAULT '1' NOT NULL,
  -- Порог (в процентах), после которого тема считается зафлуженной
  "flood_limit" smallint DEFAULT 50 NOT NULL,
  -- Период (в часах) за который выводятся темы на форуме. 0 -- выдача за все время
  "topics_period" smallint DEFAULT 0 NOT NULL,
  -- "Скрытный пользователь" (не показывать в списке присутствующих)
  "hidden" character varying(1) DEFAULT '0' NOT NULL,
  -- Режим работы визуального редактора: 0 -- выключен, 1 -- вставка тегов без визуализации, 2 -- полностью визуальный (TinyMCE)
  "wysiwyg" character varying(1) DEFAULT '1' NOT NULL,
  -- Переход после отправки сообщения: 0 -- в тему, 1 -- в раздел, 2 -- к "Обновившимся", 3 -- к "Непрочитанным"
  "goto" character varying(1) DEFAULT '0' NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_user_value
--

-- Comments:
-- Значения задаваемых полей профиля пользователя
--
CREATE TABLE "ib_user_value" (
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Идентификатор задаваемого поля
  "fdid" smallint NOT NULL,
  -- Значение задаваемого поля
  "value" character varying(255) NOT NULL,
  PRIMARY KEY ("uid", "fdid")
);

--
-- Table: ib_user_warning
--

-- Comments:
-- Предупреждения и наказания пользователей
--
CREATE TABLE "ib_user_warning" (
  "id" serial NOT NULL,
  -- Идентификатор пользователя, которому вынесено предупреждение
  "uid" integer DEFAULT 0 NOT NULL,
  -- Время вынесения
  "warntime" integer DEFAULT 0 NOT NULL,
  -- Идентификатор модератора, вынесшего предупреждение
  "moderator" integer DEFAULT 0 NOT NULL,
  -- Идентификатор сообщения, за которое вынесено предупреждение
  "pid" integer DEFAULT 0 NOT NULL,
  -- Количество штрафных баллов
  "value" smallint DEFAULT 0 NOT NULL,
  -- Дата окончания действия предупреждения
  "warntill" integer DEFAULT 0 NOT NULL,
  -- Комментарий модератора
  "descr" text NOT NULL,
  PRIMARY KEY ("id")
);

--
-- Table: ib_views
--
CREATE TABLE "ib_views" (
  -- Номер объекта
  "oid" integer NOT NULL,
  -- Для какого объекта указаны просмотры: раздел или тема
  "type" character varying(5) DEFAULT 'topic' NOT NULL,
  -- Количество просмотров
  "views" integer DEFAULT 0 NOT NULL,
  PRIMARY KEY ("oid", "type")
);

--
-- Table: ib_vote
--

-- Comments:
-- Результаты голосования отдельных пользователей
--
CREATE TABLE "ib_vote" (
  -- Номер темы с опросом
  "tid" serial NOT NULL,
  -- Идентификатор пользователя
  "uid" integer NOT NULL,
  -- Идентификатор варианта ответа
  "pvid" integer NOT NULL,
  -- Время голосования
  "time" integer NOT NULL,
  -- IP, с которого производилось голосование
  "ip" integer NOT NULL,
  PRIMARY KEY ("tid", "uid")
);

CREATE TABLE "ib_subaction" (
  "id" oid,
  "name" varchar(128) NOT NULL,
  "module" varchar(32) NOT NULL,
  "action" varchar(32) NOT NULL,
  "fid" smallint NOT NULL DEFAULT 0,
  "tid" smallint NOT NULL DEFAULT 0,
  "library" varchar(32) NOT NULL,
  "proc" varchar(32) NOT NULL,
  "block" varchar(32) NOT NULL,
  "active" char(1) NOT NULL DEFAULT '1',
  "params" varchar(255) NOT NULL,
  "priority" smallint NOT NULL,
  PRIMARY KEY ("id")
);
CREATE INDEX "intb_subaction_module_IDX" on "ib_subaction" (`module`,`action`);

INSERT INTO "ib_access" VALUES (0,0,'1','1','0','0','0','0','0','0','0','0','0','0'),(50,0,'1','1','1','0','1','0','0','0','0','0','1','1'),(100,0,'1','1','1','1','1','0','0','1','0','1','1','1'),(120,0,'1','1','1','1','1','0','0','1','0','1','1','1'),(140,0,'1','1','1','1','1','1','0','1','1','1','1','1'),(160,0,'1','1','1','1','1','1','0','1','1','1','1','1'),(180,0,'1','1','1','1','1','1','0','1','1','1','1','1'),(499,0,'1','1','1','1','1','1','0','1','1','1','1','1'),(500,0,'1','1','1','1','1','1','0','1','1','1','1','1'),(1000,0,'1','1','1','1','1','1','1','1','1','1','1','1'),(1024,0,'1','1','1','1','1','1','1','1','1','1','1','1');
INSERT INTO "ib_bots" VALUES (1,'Yandex','Яндекс',0),(2,'Googlebot','Google',0),(3,'bingbot','Bing!',0),(5,'Yahoo! Slurp','Yahoo!',0),(6,'mail.ru','@Mail.Ru',0),(7,'W3C_Validator','W3C Validator',0),(8,'SputnikBot','Sputnik',0);
INSERT INTO "ib_crontab" VALUES (1,'antibot','captcha_clear','24','Очистка старых данных CAPTCHA',0,719),(2,'maintain','log_rotate','5','Ротация логов в каталоге logs',0,1440),(3,'antibot','timeout_clear','24','Очистка старых данных о таймаутах',0,1441),(4,'maintain','search_results_clear','7','Очистка старых результатов поиска',0,1443),(5,'maintain','mod_logs_clear','90','Удаление старых данных о модераторских действиях',0,10079),(6,'maintain','light_optimize','','Малая оптимизация баз данных (только часто изменяемые таблицы)',0,4300),(7,'maintain','heavy_optimize','','Полная оптимизация базы данных (все таблицы)',0,44643),(8,'maintain','update_mark_all','90','Отметка прочитанными всех тем, которые обновились, но не были просмотрены в течение заданного количества дней',0,10081),(9,'delete','inactive_users_clear','30','Удаление пользователей, не активировавших свой профиль в течение указанного количества дней',0,4320),(10,'maintain','online_clear','3','Очистка списка последних действий пользователей',0,1440);
INSERT INTO "ib_forum_type" VALUES ('anon','Анонимный форум','1','1','1','0','0',3,'### Раздел <<<title>>> (тип anon)\r\nRewriteRule ^<<<hurl>>>/((\\d+)\\.htm)?$ anon.php?f=<<<id>>>&a=view_forum&page=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/((\\w+)\\.htm)?$ anon.php?f=<<<id>>>&a=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/((\\d+)\\.htm)?$ anon.php?f=<<<id>>>&t=$1&a=view_topic&page=$3 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/(\\w+)\\.htm$ anon.php?f=<<<id>>>&t=$1&a=$2 [L,QSA]\r\nRewriteRule ^moderate/<<<hurl>>>/((\\w+)/)?(\\w+)\\.htm$ moderate.php?f=<<<id>>>&a=$3&t=$2 [L,QSA]\r\n'),('link','Ссылка на внешний ресурс','0','0','0','0','0',4,'### Раздел <<<title>>> (тип stdforum)\r\nRewriteRule ^<<<hurl>>>/?$ link.php?f=<<<id>>>&a=view [L,QSA]\r\n'),('statpage','Статическая страница','0','1','1','1','0',2,'### Раздел <<<title>>> (тип statpage)\r\nRewriteRule ^<<<hurl>>>/((\\w+)\\.htm)?$ statpage.php?f=<<<id>>>&a=$2 [L,QSA]\r\nRewriteRule ^moderate/<<<hurl>>>/edit_foreword.htm$ statpage.php?f=<<<id>>>&a=edit [L,QSA]\r\n'),('stdforum','Обычный форум','1','1','1','1','0',1,'### Раздел <<<title>>> (тип stdforum)\r\nRewriteRule ^<<<hurl>>>/((\\d+)\\.htm)?$ stdforum.php?f=<<<id>>>&a=view_forum&page=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/((\\w+)\\.htm)?$ stdforum.php?f=<<<id>>>&a=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/((\\d+)\\.htm)?$ stdforum.php?f=<<<id>>>&t=$1&a=view_topic&page=$3 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/(\\w+)\\.htm$ stdforum.php?f=<<<id>>>&t=$1&a=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/(\\w+)\\.htm$ stdforum.php?f=<<<id>>>&t=$1&a=$2 [L,QSA]\r\nRewriteRule ^<<<hurl>>>/([\\w\\-\\d]+)/post-(\\d+)\\.htm$ stdforum.php?f=<<<id>>>&t=$1&a=post&post=$2 [L,QSA]\r\nRewriteRule ^moderate/<<<hurl>>>/(([\\w\\-\\d]+)/)?(\\w+)\\.htm$ moderate.php?f=<<<id>>>&a=$3&t=$2 [L,QSA]\r\n');
INSERT INTO "ib_group" VALUES (0,'Гость','1',90,0,2048,0,'0','0','0','0',0,'nofollow'),(50,'Сомнительный тип','1',90,3,0,0,'0','0','0','0',0,'none'),(100,'Новичок','0',30,6,1024,0,'0','0','0','0',0,'none'),(120,'Начинающий','0',30,10,1024,5,'0','0','0','0',2,'nofollow'),(140,'Участник','0',10,25,2048,25,'0','0','0','0',4,'nofollow'),(160,'Почетный участник','0',5,60,2048,100,'1','0','0','0',7,'allow'),(180,'Долгожитель форума','0',3,240,4096,500,'1','0','0','0',30,'allow'),(499,'Участник команды','1',0,0,4096,0,'1','0','1','0',0,'allow'),(500,'Модератор','1',0,0,4096,0,'1','0','1','0',0,'allow'),(1000,'Администратор','1',0,0,32767,0,'1','1','1','1',0,'nofollow'),(1024,'Создатель форума','1',0,0,32767,0,'1','1','1','1',0,'allow');
INSERT INTO "ib_menu_item" VALUES (1,1,'О проекте','about/',1,'1','1','1','1'),(2,1,'Правила','rules.htm',2,'1','1','1','1'),(3,1,'Участники','users/',5,'1','1','1','1'),(4,1,'Команда','team.htm',4,'1','1','1','1'),(5,1,'Последние сообщения','newtopics/',3,'1','1','1','1'),(6,1,'Поиск','search/',7,'1','1','1','1'),(7,1,'Справка','help/',8,'1','1','1','1'),(8,1,'Сейчас присутствуют','online/',6,'1','1','1','1');
INSERT INTO "ib_menu" VALUES (1,'Главное меню','1'),(2,'Меню Центра Администрирования','1');
INSERT INTO "ib_smile" VALUES ('8-)','cool.png','','dropdown',4),
(':(','sad.png','','dropdown',9),
(':)','smile.png','','dropdown',1),(':alien:','alien.png','','more',100),
(':angel:','angel.png','','dropdown',2),(':angry:','angry.png','','dropdown',3),
(':blink:','blink.png','','more',101),(':blush:','blush.png','','more',102),(':cheerful:','cheerful.png','','more',103),(':D','grin.png','','dropdown',7),(':devil:','devil.png','','more',104),(':dizzy:','dizzy.png','','more',105),(':ermm:','ermm.png','','dropdown',6),(':face:','face.png','','more',119),(':getlost:','getlost.png','','more',106),(':happy:','happy.png','','more',107),(':kissing:','kissing.png','','more',108),(':laughing:','laughing.png','','more',120),(':love:','wub.png','','hidden',501),(':ninja:','ninja.png','','more',109),(':O','shocked.png','','dropdown',10),(':P','tongue.png','','dropdown',11),(':pinch:','pinch.png','','more',110),(':pouty:','pouty.png','','more',111),(':sick:','sick.png','','more',112),(':sideways:','sideways.png','','more',113),(':silly:','silly.png','','more',114),(':sleeping:','sleeping.png','','more',115),(':unsure:','unsure.png','','more',116),(':wassat:','wassat.png','','more',118),(':whistling:','whistling.png','','hidden',500),(':woot:','w00t.png','','more',117),(';)','wink.png','','dropdown',12),('<3','heart.png','','dropdown',8);
INSERT INTO "ib_text" VALUES (0,0,'Правила форума разрабатываются. А пока просим придерживаться общих принципов вежливости и доброжелательности.',0),(1,2,'Если вы читаете этот текст, то установка Intellect Board прошла успешно. \r\nВ дальнейшем его можно будет заменить на информацию о вашем проекте или просто удалить.\r\nЭтот раздел имеет тип \"Статическая страница\". Обычный раздел с темами и соощениями вы можете \r\nсоздать в Центре Администрирования.',0);
INSERT INTO "ib_user" VALUES (1,'Guest','*',1,'','U',NULL,' ',0,'guest','',111,'Гость','none','none','null@intbpro.ru',''),(2,'System','*',1,'','U',NULL,'',0,'system','',222,'System','none','none','null@intbpro.ru',''),(3,'New User','*',1,'','U',NULL,'',0,'NewUser','',333,'New User','none','none','null2@intbpro.ru','');
INSERT INTO "ib_user_contact_type" VALUES (2,'Skype','icons/c/skype.gif','skype:%s',50,'','0'),(3,'ВКонтакте','icons/c/vk.gif','http://vk.com/%s',30,'vkontakte','0'),(4,'ICQ','icons/c/icq.gif','',20,'','0'),(5,'Jabber/XMPP','icons/c/jabber.gif','xmpp:%s',100,'','0'),(6,'МойМир@Mail.Ru','icons/c/agent.gif','http://my.mail.ru/%s',60,'mailru','0'),(7,'LiveJournal','icons/c/lj.gif','http://%s.livejournal.com',70,'livejournal','0'),(8,'Яндекс.Фотки','icons/c/yandex.gif','http://fotki.yandex.ru/users/%s',80,'yandex','0'),(9,'GTalk/GMail','icons/c/gtalk.gif','mailto:%s@gmail.com',40,'google','0'),(10,'Одноклассники','icons/c/odno.gif','http://www.odnoklassniki.ru/profile/%s',35,'odnoklassniki','0'),(11,'Facebook','icons/c/facebook.gif','https://www.facebook.com/profile.php?id=%s',37,'facebook','0'),(12,'Twitter','icons/c/twitter.gif','http://twitter.com/%s',90,'twitter','0'),(13,'Webmoney ID','icons/c/webmoney.gif','https://passport.webmoney.ru/asp/CertView.asp?wmid=%s',120,'webmoney','0'),(14,'OpenID','icons/c/openid.gif','%s',110,'openid','0'),(15,'Личный сайт','','%s',100,'','1'),(16,'Личный блог','','%s',100,'','1');
INSERT INTO "ib_user_ext" VALUES (1,10,0,0,0,0,0,1411401372,0),(2,0,0,0,0,0,0,1411401372,0),(3,0,0,0,0,0,100,1411401372,0);
INSERT INTO "ib_user_settings" VALUES (1,0,0,'','ASC','None',10800,'1','1','1','1','0','0',1,'1','1','1','1',50,0,'0','1','0'),(3,15,20,'','ASC','My',10800,'1','1','1','1','0','0',1,'1','1','1','1',50,0,'0','2','0');



CREATE OR REPLACE FUNCTION substring_index (
  str text,
  delim text,
  count integer = 1,
  out substring_index text
)
RETURNS text AS
$body$
BEGIN
  IF count > 0 THEN
    substring_index = array_to_string((string_to_array(str, delim))[:count], delim);
  ELSE
    DECLARE
      _array TEXT[];
    BEGIN
      _array = string_to_array(str, delim);
      substring_index = array_to_string(_array[array_length(_array, 1) + count + 1:], delim);    
    END;  
  END IF;
END;
$body$
LANGUAGE 'plpgsql'
IMMUTABLE
CALLED ON NULL INPUT
SECURITY INVOKER
COST 5;

CREATE OR REPLACE FUNCTION FROM_UNIXTIME(unixtime bigint) RETURNS timestamp with time zone  AS 'SELECT to_timestamp(unixtime)' LANGUAGE SQL